/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PAPERMAP_H
#define PAPERMAP_H

#include "commdefs.h"
#include "commtypes.h"
#include "refcounter.h"

//#ifdef QT_OPENGL_LIB
//#   include "../../../glew/include/GL/glew.h"

#include "iArray.h"
#include "iArrayFilter.h"
#include "iPaperSwitch.h"
#include "qhasmath.h"



/**
 * A paper texture descriptor.
 */
class Papermap : public CRefCounter {
protected:
    /**
     * User-defined
     * paper texture.
     */
    sPaper * m_Paper;               /**< Original height map.   */
    IArray * m_Bumpmap;             /**< Normals map.           */

    IOpenGLHelper * m_Helper;       /**< Helper object.         */

    int   m_Dpi;                    /**< Paper dpi.             */

    void generateBump (void);

    virtual void suicide (void) {
        delete this;
    }

public:
    Papermap (IOpenGLHelper * helper, int dpi);
    virtual ~Papermap (void);

    /**
     * Set the new paper descriptor.
     */
    void setPaper (sPaper * map);

    /**
     * Get the current paper descriptor.
     */
    sPaper * paper (void) {
        return m_Paper;
    }

    IArray * original (void) const {
        return m_Paper->height;
    }

    IArray * detail (float detailLevel) const;

    IArray * bump (void) const;

    int paperDpi (void) const {
        return m_Dpi;
    }
};

#endif // PAPERMAP_H
