/***************************************************************************
 *   Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>              *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "pigment_xml.h"


QHash<QString, QList<IPigmentDescription*>*> * CQXmlHandler_Pigment::pigments (void) {
    return &packs;
}

void CQXmlHandler_Pigment::clear (void) {
    SAFE_RELEASE (cur_pig);

    foreach (QList<IPigmentDescription*> * list, packs) {
        for (INT i = 0; i < list->size (); i++) {
            list->at (i)->release ();
        }

        SAFE_DELETE (list);
    }

    packs.clear ();
}

CQXmlHandler_Pigment::CQXmlHandler_Pigment (void) {
    format   = 0;
    cur_list = NULL;
    cur_pig  = NULL;
}

CQXmlHandler_Pigment::~CQXmlHandler_Pigment (void) {
    clear ();
}

bool CQXmlHandler_Pigment::startElement (
                                const QString &,
                                const QString &,
                                const QString & name,
                                const QXmlAttributes & attributes) {

     if (name == "format") {
         // Format descriptor.
         //
         bool good;
         INT type = attributes.value ("type").toInt (&good, 10);
         if (good)
             format = type;
     } else if (name == "pigments_pack") {
         // Pigments pack begins here
         //
         QString name = attributes.value ("name");
         if (!name.isEmpty ()) {
             // Create new hash entry
             cur_list = new QList<IPigmentDescription*> ();
             packs.insert (name, cur_list);
         }
     } else if (name == "pigment") {
         // Pigment declaration begins here
         //
         QString name = attributes.value ("name");
         if (!name.isEmpty ()) {
             if (cur_pig) {
                 cur_pig->release ();
                 cur_pig = NULL;
             }

             cur_pig = g_fac_SPAWN_PIGMENT_DESCRIPTION ();
             cur_pig->setName (name.toUtf8 ().data ());
         }
     } else if (name == "phisical") {
         if (cur_pig) {
            // Here comes the phisical properties
            // of pigment
            //
            FLOAT vals[] = {0.5f, 0.5f, 0.5f, 0.1f};
            QString names[] = {
                "granulation",
                "density",
                "staining_power",
                "lightfastness"
            };

            for (INT i = 0; i < 4; i++) {
                bool good;
                float val = (FLOAT) attributes.value (names[i]).toFloat (&good);
                if (good)
                    vals[i] = val;
            }

            cur_pig->setWeight        (vals[1]);
            cur_pig->setGranulation   (vals[0]);
            cur_pig->setStainingPower (vals[2]);
            cur_pig->setLightfastness (vals[3]);
         }
     } else if (name == "absorbing") {
         if (cur_pig) {
            // Here comes the optical properties
            // of pigment
            //
            float vals[] = {0.0f, 0.0f, 0.0f, 0.0f};
            QString names[] = {
                "r", "g", "b", "",
            };

            for (INT i = 0; i < 3; i++) {
                bool good;
                float val = (FLOAT) attributes.value (names[i]).toFloat (&good);
                if (good)
                    vals[i] = val;
            }

            cur_pig->setAbsorbing  (vals);
         }
     } else if (name == "scattering") {
         if (cur_pig) {
            // Here come the optical properties
            // of pigment
            //
            float vals[] = {0.0f, 0.0f, 0.0f, 0.0f};
            QString names[] = {
                "r", "g", "b", "",
            };

            for (INT i = 0; i < 3; i++) {
                bool good;
                float val = (FLOAT) attributes.value (names[i]).toFloat (&good);
                if (good)
                    vals[i] = val;
            }

            cur_pig->setScattering (vals);
         }
     }

    return TRUE;
}

bool CQXmlHandler_Pigment::endElement (
                                const QString &,
                                const QString &,
                                const QString & name) {

    if (name == "pigment") {
        // Pigment declaration ends here
        //
        if (cur_list && cur_pig)
            cur_list->push_front (cur_pig);

        cur_pig = NULL;
    } else if (name == "pigments_pack") {
        // Pigments pack is filled
        //
        cur_list = NULL;
        cur_pig  = NULL;
    }

    return TRUE;
}

bool CQXmlHandler_Pigment::characters (const QString &) {
    // O.o
    return TRUE;
}

bool CQXmlHandler_Pigment::fatalError (const QXmlParseException &) {
    // O.o
    return TRUE;
}

QString CQXmlHandler_Pigment::errorString (void) const {
    return "o.O ... I know nothing.";
}
