/***************************************************************************
 *   Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>              *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#pragma once

#include "commdefs.h"
#include "commtypes.h"
#include "refcounter.h"

#include <QList>
#include <QHash>
#include <QXmlDefaultHandler>

#include "iPigmentDescription.h"


/**
 * Hash table of named
 * pigment packs.
 */
typedef QHash<QString, QList<IPigmentDescription*>*> QPIGMENTSHASH;

/**
 * Handler for xml-elements from pigments list.
 * Generates the QList object of IPigment pointers.
 */
class CQXmlHandler_Pigment : public QXmlDefaultHandler {
    IPigmentDescription         * cur_pig;
    QList<IPigmentDescription*> * cur_list;
    QPIGMENTSHASH packs;

    /**
     * Format of the input file.
     */
    INT format;
public:

     CQXmlHandler_Pigment (void);
    ~CQXmlHandler_Pigment (void);

    /**
     * Returns the hash table of
     * loaded pigments packs.
     */
    QPIGMENTSHASH * pigments  (void);

    /**
     * Delete all loaded data.
     */
    void clear (void);

    bool startElement (const QString &namespaceURI, const QString &localName,
                       const QString &qName, const QXmlAttributes &attributes);

    bool endElement (const QString &namespaceURI, const QString &localName,
                     const QString &qName);

    bool characters (const QString &str);

    bool fatalError (const QXmlParseException &exception);

    QString errorString (void) const;
};
