#include "controlsstack.h"

ControlsStack::ControlsStack (QWidget * parent) : QFrame (parent) {
    m_Id = ISpawnedObject::allocateId ();

    m_Error     = NULL;
    m_Logger    = NULL;
    m_Box0      = new QVBoxLayout;
    m_Box1      = new QVBoxLayout;
    m_Scroll    = new QScrollArea (this);
    m_Container = new QWidget (this);

    m_LoggingEnabled = FALSE;

//     QFont font;
//     font.setPointSize (8);

    m_Spacer = new QWidget  (this);
    m_Spacer->setSizePolicy (QSizePolicy::Preferred, QSizePolicy::Minimum);

//     this->setFont         (font);
    this->setLayout       (m_Box0);
//     this->setMinimumSize  (256, 512);
//     this->setMaximumWidth (256);
    this->setWindowFlags  (Qt::Tool);

    m_Container->setLayout      (m_Box1);
    m_Container->setMinimumSize (240, 640);
    m_Container->setMaximumSize (240, 2048);
    m_Container->setSizePolicy (QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);

    m_Scroll->setWidget                    (m_Container);
    m_Scroll->setBackgroundRole            (QPalette::Dark);
    m_Scroll->setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
    m_Scroll->setVerticalScrollBarPolicy   (Qt::ScrollBarAsNeeded);
    m_Scroll->setWidgetResizable           (FALSE);

    m_Box0->addWidget (m_Scroll);
    m_Box1->addWidget (m_Spacer);

    m_Box0->setContentsMargins (0, 0, 0, 0);
    m_Box1->setContentsMargins (0, 0, 0, 0);
}

ControlsStack::~ControlsStack (void) {
    m_Error = NULL;
    //SAFE_DELETE (m_Spacer);
    //SAFE_DELETE (m_Scroll);
    //SAFE_DELETE (m_Box0);
    //SAFE_DELETE (m_Box1);
    //SAFE_DELETE (m_Container);

    foreach (sItem * item, m_Items) {
        SAFE_DELETE (item->button);
        SAFE_DELETE (item);
    }
}

void ControlsStack::push (QWidget * widget, QString name) {
    m_Error = NULL;

    sItem * item = new sItem;
    if (item) {
        // Init the new item.
        item->widget = widget;
        item->button = new QPushButton (name, this);
        item->button->setCheckable (TRUE);
        //item->button->setIcon (QIcon (":/res/triangle-open.png"));
        item->button->setSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::Maximum);
        //item->widget->setSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        item->widget->setVisible (TRUE);

        // Add the widget block
        // into the layout.
        m_Box1->insertWidget (m_Box1->count () - 1, item->button, 0, 0);
        m_Box1->insertWidget (m_Box1->count () - 1, item->widget, 0, 0);

        connect (item->button, SIGNAL(toggled(bool)), this, SLOT(triggered(bool)));
        m_Items.append (item);
    }

    triggered (TRUE);
}

void ControlsStack::showWidget (QWidget * widget) {
    foreach (sItem * item, m_Items) {
        if (item->widget == widget) {
            item->button->setChecked (TRUE);
            item->widget->setVisible (TRUE);
            break;
        }
    }
}

void ControlsStack::hideWidget (QWidget * widget) {
    foreach (sItem * item, m_Items) {
        if (item->widget == widget) {
            item->button->setChecked (FALSE);
            item->widget->setVisible (FALSE);
            break;
        }
    }
}

void ControlsStack::triggered (bool checked) {
    Q_UNUSED (checked);

    foreach (sItem * item, m_Items) {
        if (item->button->isChecked ()) {
            item->widget->setVisible (TRUE);
        } else {
            item->widget->setVisible (FALSE);
        }
    }
}
