/* Qaquarelle
 * Copyright (C) 2009 Vasiliy Makarov <drmoriarty.0@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "paintingstyle.h"

static const int TemplatesCount = 6;
static const PaintingStyleSettings Templates[TemplatesCount] = {
    PaintingStyleSettings(QObject::tr("Custom"),        PAINTINGSTYLE_CUSTOM),
    PaintingStyleSettings(QObject::tr("Plain"),         PAINTINGSTYLE_PLAIN),
    PaintingStyleSettings(QObject::tr("Watercolours"),  PAINTINGSTYLE_WATERCOLOURS),
    PaintingStyleSettings(QObject::tr("Acrilycs"),      PAINTINGSTYLE_ACRILYCS),
    PaintingStyleSettings(QObject::tr("Oil"),           PAINTINGSTYLE_OIL),
    PaintingStyleSettings(QObject::tr("Ink"),           PAINTINGSTYLE_INK)
};


_PaintingStyleSettings::_PaintingStyleSettings(QString n, ePAINTINGSTYLE_PRESET p)
	: name(n), preset(p)
{
}

PaintingStyle::PaintingStyle(QWidget *parent) : QWidget(parent), m_PaintType(0), old_style(-1)
{
    ui.setupUi(this);
    for(int i=0; i<TemplatesCount; i++) {
        ui.comboBox->addItem (Templates[i].name);
    }
    ui.comboBox->setCurrentIndex(2); // watercolours
    //setTemplate(2); // watercolours

    // [castedflake] Let's make this panel floating, huh?
    ui.frame->setWindowFlags (Qt::Popup | Qt::Tool);

    connect (ui.toolButton, SIGNAL(toggled(bool)), this, SLOT(showCustomFrame(bool)));
    connect (ui.frame, SIGNAL(closed()), ui.toolButton, SLOT(toggle()));

    lock = FALSE;
}

PaintingStyle::~PaintingStyle()
{
}

void PaintingStyle::showCustomFrame (bool on) {
    if (on) {
        QPoint p = mapToGlobal (QPoint (0, 0));
        p.setY (p.y() + height () + 8);

        ui.frame->move (p);
        ui.frame->show ();
    } else {
        ui.frame->hide ();
    }
}

void PaintingStyle::setPaintingStyle (IPaintingStyle * paint)
{
	m_PaintType = paint;
}

void PaintingStyle::setDryingSpeed(float f)
{
    if(m_PaintType && !lock) {
		m_PaintType->setDryingSpeed(f);
        ui.comboBox->setCurrentIndex (0);
        //emit paintingStyleChanged();
	}
}

void PaintingStyle::setOpacity(float f)
{
    if(m_PaintType && !lock) {
		m_PaintType->setOpacity(f);
        ui.comboBox->setCurrentIndex (0);
        //emit paintingStyleChanged();
	}
}

void PaintingStyle::setStainingPower(float f)
{
    if(m_PaintType && !lock) {
		m_PaintType->setStainingPower(f);
        ui.comboBox->setCurrentIndex (0);
        //emit paintingStyleChanged();
	}
}

void PaintingStyle::setViscosity(float f)
{
    if(m_PaintType && !lock) {
		m_PaintType->setViscosity(f);
        ui.comboBox->setCurrentIndex (0);
        //emit paintingStyleChanged();
	}
}

void PaintingStyle::setTemplate(int i)
{
    // [castedflake] Made the button always shown.
    showCustomFrame           (Templates[i].preset < 0);
    ui.toolButton->setChecked (Templates[i].preset < 0);

    if(m_PaintType && Templates[i].preset >= 0) {
        //m_PaintType->fromPreset(i);
        m_PaintType->setDryingSpeed(IPigmentDescription::dryingspeed(Templates[i].preset));
        m_PaintType->setOpacity(IPigmentDescription::opacity(Templates[i].preset));
        m_PaintType->setStainingPower(IPigmentDescription::stainingPower(Templates[i].preset));
        m_PaintType->setViscosity(IPigmentDescription::viscosity(Templates[i].preset));

        // Update the bars.
        lock = TRUE;
        ui.bar_dryingSpeed->setPosition (m_PaintType->paintDryingSpeed ());
        ui.bar_opacity->setPosition (m_PaintType->paintOpacity ());
        ui.bar_stainingPower->setPosition (m_PaintType->paintStainingPower ());
        ui.bar_viscosity->setPosition (m_PaintType->paintViscosity ());
        lock = FALSE;
    }

    emit paintingStyleChanged();
}

void PaintingStyle::setBrushGroup(bool b)
{
	if(b) {
		if(!ui.comboBox->isEnabled()) {
			ui.comboBox->setDisabled(false);
                        ui.toolButton->setDisabled(false);
			ui.comboBox->setCurrentIndex(old_style);
		}
	} else {
		if(ui.comboBox->isEnabled()) {
			old_style = ui.comboBox->currentIndex();
			ui.comboBox->setCurrentIndex(1);
			ui.comboBox->setDisabled(true);
                        ui.toolButton->setDisabled(true);
		}
	}
}

void PaintingStyle::sync (ePAINTINGSTYLE_PRESET preset) {
    ui.comboBox->setCurrentIndex ((int)preset + 1);

    showCustomFrame           (FALSE);
    ui.toolButton->setChecked (FALSE);
}

void PaintingStyle::moveEvent (QMoveEvent * event) {
    Q_UNUSED (event);
    if (ui.frame->isVisible ()) {
        // Move the popup frame with the widget.
        QPoint p = mapToGlobal (QPoint (0, 0));
        p.setY (p.y() + height () + 8);

        ui.frame->move (p);
    }
}
