#include "palette.h"
//#include "pigmenteditor.h"
#include <stdio.h>
#include <QFileDialog>
#include "iPigmentDescription.h"

IPalette * g_fac_SPAWN_PALETTE (QWidget * parent, ISkin * skin, bool isstatic) {
    return new Palette (parent, skin, isstatic);
}

IPalette * g_fac_SPAWN_PALETTE (void) {
    return new Palette (NULL, NULL, TRUE);
}

Palette::Palette (QWidget * parent, ISkin * skin, bool isstatic) : QFrame (parent) {
    m_IsDynamic = !isstatic;

    ui.setupUi (this);

    m_Id           = ISpawnedObject::allocateId ();
    m_Orientation  = TRUE;
    m_Hidable      = TRUE;
    m_Length       = 16;
    m_MaxDynColors = 32;
    m_Connector    = NULL;
    m_Skin         = skin;
    m_PaintMode    = PAINT_KM;
    if (skin) {
        skin->addRef ();
    }

    m_Grid = new QGridLayout (ui.framePigments);
    ui.framePigments->setLayout (m_Grid);
    ui.scrollArea->setContentsMargins (0, 0, 0, 0);

    if (m_IsDynamic) {
        ui.pushSave->setVisible (FALSE);
        ui.pushLoad->setVisible (FALSE);
        ui.comboSorting->setVisible (FALSE);
    }

    QObject::connect (ui.pushSave,     SIGNAL(released()), this, SLOT(savePalette()));
    QObject::connect (ui.pushLoad,     SIGNAL(released()), this, SLOT(loadPalette()));
    QObject::connect (ui.comboSorting, SIGNAL(currentIndexChanged(int)), this, SLOT(sortingChanged(int)));

    setWindowTitle (tr ("Palette"));
    setMaximumSize (1000, 1000);
    setSizePolicy  (QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
    setContentsMargins (0, 0, 0, 0);
}

Palette::~Palette (void) {
    clear ();
    SAFE_RELEASE (m_Connector);
    SAFE_RELEASE (m_Skin);
}

void Palette::poke (IPigmentMixture * pig, float amount) {
    emit touched (pig, amount, COLORMETHOD_PALETTE);
}

void Palette::editPigment (PigmentBox * box) {
    // FIXME Disabled now.
    if (box) {
        /*
        PigmentEditor dlg (this);

        dlg.init (box->pigment ());

        // Show the dialog and set the
        // resulting pigment to the box.
        int result = dlg.exec ();
        if (result) {
            IPigmentMixture * pig = dlg.pigment ();
            if (pig) {
                box->setPigment (pig);
                pig->release ();
            }
        }
        */
    }
}

/// Ask the filename and save.
void Palette::savePalette (void) {
    QString fileName = QFileDialog::getSaveFileName (
        this,
        QString (),                                                                     // Caption
        QString (),                                                                     // Dir
        tr ("XML files (*.xml)"),                                                       // Filter
        NULL,                                                                           // Selected filter
        0                                                                               // Options
    );
    if (!fileName.isEmpty ()) {
        // Auto-append extension
        if (QFileInfo (fileName).suffix ().isEmpty ()) {
            fileName += ".xml";
        }

        if (FALSE == save (fileName.toUtf8 ().data ())) {
            QMessageBox::warning (this,
                                  tr ("Qaquarelle saving error."),
                                  tr ("Failed to save the palette... =(")
            );
        }
    }
}

/// Ask the filename and load.
void Palette::loadPalette (void) {
    QString fileName = QFileDialog::getOpenFileName (
        this,
        QString (),                                                                     // Caption
        QString (),                                                                     // Dir
        tr ("XML files (*.xml)"),                                                       // Filter
        NULL,                                                                           // Selected filter
        0                                                                               // Options
    );
    if (!fileName.isEmpty ()) {
        // Auto-append extension
        if (QFileInfo (fileName).suffix ().isEmpty ()) {
            fileName += ".xml";
        }

        if (FALSE == load (fileName.toUtf8 ().data ())) {
            QMessageBox::warning (this,
                                  tr ("Qaquarelle loading error."),
                                  tr ("Failed to load the palette from your file... =(")
            );
        }
    }
}

void Palette::clear (void) {
    foreach (QWidget * c, m_Colours) {
        if (c) delete c;
    }

    m_Colours.clear ();
}
/*
void Palette::flip (void) {
    setVertical (!vertical ());
    rebound ();
}
*/
bool Palette::load (const char * path) {
    bool result = FALSE;
    if (path && m_Connector) {
        FILE * f = fopen (path, "rb");
        if (f) {
            fseek (f, 0, 2);
            long sz = ftell (f);
            fseek (f, 0, 0);

            if (sz) {
                BYTE * data = new BYTE[sz];
                if ((sz = fread (data, 1, sz, f))) {
                    // Clear the palette.
                    clear ();

                    // Call the parser.
                    m_Connector->parsePigmentsPack (this, 13, data, sz);
                    result = TRUE;
                }

                SAFE_DELETE_A (data);
            }
            fclose (f);
        }
    }

    return result;
}

bool Palette::save (const char * path) {
    bool result = FALSE;
    if (path) {
        FILE * f = fopen (path, "wb");
        if (f) {
            char buffer[1024];
            fprintf (f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
            fprintf (f, "<pigbase>\n\t<format type=\"0\" />\n\t\t");
            fprintf (f, "<pigments_pack name=\"palette\">\n\n");

            // Save all pigments.
            for (int i = 0; i < m_Colours.size (); i++) {
                long sz = m_Colours[i]->pigment ()->toXML (buffer, 1024);
                fwrite (buffer, sz, 1, f);
            }

            fprintf (f, "\t</pigments_pack>\n</pigbase>\n");
            fclose (f);
            result = TRUE;
        }
    }

    return result;
}

void Palette::addPigment (IPigmentMixture * pigment) {
    PigmentBox * box = new PigmentBox (this, m_Skin);
                 box->setPigment (pigment);

    // Connect the color-box to palette.
    QObject::connect (box, SIGNAL (touched(IPigmentMixture*, float)), this, SLOT (poke(IPigmentMixture*, float)));
    QObject::connect (box, SIGNAL (editPigmentRequest(PigmentBox*)),  this, SLOT (editPigment(PigmentBox*)));

    m_Colours.prepend (box);
    rebound ();
    update ();
}

void Palette::rebound (void) {
    while (m_Grid->count ()) {
        m_Grid->removeItem (m_Grid->itemAt (0));
    }

    if (m_Orientation) {
        m_Length = ui.scrollArea->width() / 22 - 1;
    } else {
        m_Length = ui.scrollArea->height() / 22 - 1;
    }

    if (m_Length < 2)\
        m_Length = 2;

    int maxx = 0;
    int maxy = 0;
    for (int i = 0; i < m_Colours.size (); i++) {
        PigmentBox * c = m_Colours.at (i);
        int x, y;
        if (!m_Orientation) {
            x = i / m_Length;
            y = i % m_Length;
        } else {
            y = i / m_Length;
            x = i % m_Length;
        }

        m_Grid->addWidget (c, y, x, Qt::AlignCenter);
        c->show ();
        c->setVertical (m_Orientation);

        maxx = MAX (maxx, x);
        maxy = MAX (maxy, y);
    }
/*
    if (m_Orientation) {
        ui.framePigments->setMinimumSize  ((maxx + 1) * 22, (maxy + 1) * 32);
        ui.framePigments->setMaximumSize  ((maxx + 1) * 22, (maxy + 1) * 32);
    } else {
        ui.framePigments->setMinimumSize  ((maxx + 1) * 32, (maxy + 1) * 22);
        ui.framePigments->setMaximumSize  ((maxx + 1) * 32, (maxy + 1) * 22);
    }
*/
}

void Palette::connect (IConnector * caller) {
    SAFE_RELEASE (m_Connector);

    m_Connector = caller;
    if (caller) {
        caller->addRef ();
        caller->allocateObject (this, 13, ENTITY_PIGMENT);
    }
}

void Palette::complitedRequest (int param, int length, void * data) {
    Q_UNUSED (length);
    if (param == 13) {
        IPigmentMixture * pig = static_cast<IPigmentMixture*>(data);
        if (pig) {
            addPigment (pig);
            pig->release ();
        }
    }
}

void Palette::resizeEvent ( QResizeEvent * event )
{
    QFrame::resizeEvent(event);

    // Set the orientation.
    setVertical (event->size ().width  () <
                 event->size ().height ());
    rebound();
}

void Palette::showEvent (QShowEvent * event) {
    ui.pushSave->setVisible     (!m_IsDynamic);
    ui.pushLoad->setVisible     (!m_IsDynamic);
    ui.comboSorting->setVisible (!m_IsDynamic);
}

void Palette::insertCopyOfPigment (IPigmentMixture * pigment)
{
    while (m_Colours.size () >= m_MaxDynColors) {
        delete m_Colours.back ();
        m_Colours.pop_back ();
    }

    // [castedflake] Search for duplication.
    bool duplicate = FALSE;
    foreach (QWidget * c, m_Colours) {
        PigmentBox * pig = dynamic_cast<PigmentBox*>(c);
        const float * a = pig->pigment ()->absorbing  ();
        const float * s = pig->pigment ()->scattering ();

        float d = fabsf (a[0] - pigment->absorbing()[0]) +
                  fabsf (a[1] - pigment->absorbing()[1]) +
                  fabsf (a[2] - pigment->absorbing()[2]) +
                  fabsf (s[0] - pigment->scattering()[0]) +
                  fabsf (s[1] - pigment->scattering()[1]) +
                  fabsf (s[2] - pigment->scattering()[2]);

        if (!IS_SIGNIFICANT (d)) {
            duplicate = TRUE;
            break;
        }
    }

    // Add if not found.
    if (!duplicate) {
        IPigmentDescription * pigment1 = g_fac_SPAWN_PIGMENT_DESCRIPTION ();
        pigment1->setAbsorbing(pigment->absorbing());
        pigment1->setScattering(pigment->scattering());
        pigment1->setName(pigment->name());
        pigment1->setWeight(pigment->weight());
        pigment1->setGranulation(pigment->granulation());
        pigment1->setStainingPower(pigment->stainingPower());
        //m_Pigment->setLightfastness(pigment->lig);
        addPigment(pigment1);
        pigment1->release ();
    }
}

void Palette::sort (ePaletteSortIndex index, bool asc) {
    if (PALETTE_DONT_SORT == index)
        return;

    // Selection sort.
    for (int i = 0; i < m_Colours.size () - 1; i++) {
        int sel = i;
        for (int j = i + 1; j < m_Colours.size (); j++) {
            if ((PALETTE_SORT_BY_HUE == index &&
                (m_Colours[j]->pigment()->sortIndexHue () >
                 m_Colours[i]->pigment()->sortIndexHue ()) == asc) ||
                (PALETTE_SORT_BY_BRIGHTNESS == index &&
                (m_Colours[j]->pigment()->sortIndexBrightness () >
                 m_Colours[i]->pigment()->sortIndexBrightness ()) == asc))
                sel = j;
        }

        m_Colours.swap (i, sel);
    }

    rebound ();
}

void Palette::sortingChanged (int idx) {
    if (idx >= 0) {
        sort ((ePaletteSortIndex) idx, TRUE);
    }
}
