/***************************************************************************
 *   Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>              *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IWORK_H
#define IWORK_H

#include "commtypes.h"
#include "commdefs.h"
#include "refcounter.h"

//#ifdef QT_OPENGL_LIB
#   include "../../glew/include/GL/glew.h"

#include "iConnector.h"


class IWork : public IConnectorCallback,
              public IVerboseObject {
public:
    /**
     * Perform the work.
     * @returns TRUE if the work is not idling.
     */
    virtual bool execute (void) = 0;

    /**
     * Release all data.
     */
    virtual void release (void) = 0;

    /**
     * Get the task priority.
     */
    virtual int priority (void) const = 0;

    /**
     * Get the task persistence flag.
     */
    virtual bool isPersisitent (void) const = 0;

    /**
     * Check if the task is active now.
     */
    virtual bool isActive (void) const = 0;

    /**
     * Activate or deactivate the task.
     */
    virtual void setActive (bool state) = 0;

    /**
     * Get the task execution interval in ms.
     */
    virtual long period (void) const = 0;
};

#endif // IWORK_H
