/***************************************************************************
 *   Copyright (C) 2009                                                    *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui/QApplication>
#include <QFile>
#include "mainwindow.h"
#include "preferences.h"

Q_IMPORT_PLUGIN (brushes)
Q_IMPORT_PLUGIN (basicdynamics)


int main (int argc, char *argv[]) {
    PROFILE_FUNC();
    QCoreApplication::setOrganizationName("FreePainters");
    //QCoreApplication::setOrganizationDomain("");
    QCoreApplication::setApplicationName("qAquarelle");
	
    QApplication a (argc, argv);
  //QApplication::setStyle (QStyleFactory::create ("White"));

    QString locale = QLocale::system().name().left(2);
    QTranslator translator;
    QString loc_path;
#ifndef _WIN32
    loc_path = qApp->applicationDirPath() + "/locale/";
#endif
    loc_path += locale;
    translator.load("qaquarelle", loc_path);
    qDebug("Loading l10n from %s", (const char*)loc_path.toLocal8Bit());
    a.installTranslator(&translator);

    // Open the log file.
    QDir::home().mkdir(".qaquarelle");
    QFile * logFile = new QFile (QDir::homePath()+"/.qaquarelle/qaquarelle-client.log");
    if (logFile) {
        logFile->open (QIODevice::WriteOnly);
    }

    // Create a logger.
    Logger_signal * logger = new Logger_signal ();

    // And attach it to a writer.
    Logfile_helper * writer = new Logfile_helper (logFile);
    QObject::connect (logger, SIGNAL (message(eLOG_IMPORTANCE,const char*)),
                      writer, SLOT   (write(eLOG_IMPORTANCE,const char*)));

    int status = 0;
    MainWindow * w = new MainWindow (NULL, logger);
    if (w) {
	    QSettings set;
		if(set.contains("winstate")) w->restoreState(set.value("winstate").toByteArray(), 1);
		if(set.contains("wingeometry")) w->restoreGeometry(set.value("wingeometry").toByteArray());
        w->show ();

        // And run our great application... x_x
        status = a.exec ();

		// save window state
		set.setValue("winstate", w->saveState(1));
		set.setValue("wingeometry", w->saveGeometry());
        SAFE_DELETE (w);
    }

    PREF.save();

    // Check for undeleted objects.
    if (ISpawnedObject::totalObjects ()) {
        logger->write (LOGIMPORTANCE_WARNING, NULL, "Detected %d undeleted objects.",
                       ISpawnedObject::totalObjects ());

        ISpawnedObject * orphan = ISpawnedObject::orphans ();
        while (orphan) {
            // Report the object.
            logger->write (LOGIMPORTANCE_WARNING, NULL,
                           "Undeleted object of class %s with id %d!",
                           orphan->myClassName (), orphan->objectId ());

            orphan = orphan->next ();
        }

        // Try to release them all.
        while (ISpawnedObject::orphans ())
               ISpawnedObject::orphans ()->release ();
    }

    logger->write (LOGIMPORTANCE_NEVERMIND, NULL, "Successfully terminated.");

    // Release logger and helper.
    SAFE_RELEASE (logger);
    SAFE_DELETE  (writer);

    if (logFile) {
        logFile->close ();
        SAFE_DELETE (logFile);
    }
#ifdef DEBUG
	PROFILER_UPDATE();
	PROFILER_OUTPUT("/tmp/qaquarelle-profile.log");
#endif

    return status;
}
