/***************************************************************************
 *   Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>              *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SKIN_H
#define SKIN_H

#include "commtypes.h"
#include "commdefs.h"
#include "refcounter.h"

#include "iSkin.h"
#include <QHash>


class Skin : public ISkin,
             public CRefCounter {
public:
    /******* ISKIN IMPLANTS **********/

    /**
     * Set the skin root directory.
     */
    virtual void setRoot (QString path);

    /**
     * Returns the full path to the skin image
     * with the given name.
     */
    virtual QString pixmapPathByName (QString name) const;

    /**
     * Load an image from the skin data
     * by its name. Images are cached.
     */
    virtual QImage * pixmapByName (QString name);

    /******** ISPAWNEDOBJECT IMPLANTS ************/

    /**
     * Get the classname for this object.
     */
    virtual const char * myClassName (void) {
        return "Client::Skin";
    }

    /**
     * Get the Id of current allocated object.
     */
    virtual int objectId (void) {
        return m_Id;
    }

    /**
     * Increase the object's
     * reference counter.
     */
    virtual void addRef (void) {
        CRefCounter::addRef ();
    }

    /**
     * Release the object.
     */
    virtual void release (void) {
        CRefCounter::release ();
    }

    /**
     * Ask the object to emit all
     * notification signals if any.
     */
    virtual void pokeObject (void) {
    }

friend ISkin * g_fac_SPAWN_SKIN (QString path);
private:
    int         m_Id;
    QDir        m_SkinRoot;

    /**
     * A hashed list of loaded images.
     */
    QHash<QString, QImage*> m_HashedImages;

    void clean (void);

    Skin (QString path);
    virtual ~Skin (void);

    virtual void suicide (void) {
        delete this;
    }
};

#endif // SKIN_H
