/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once
#include "commdefs.h"
#include "commtypes.h"
#include "refcounter.h"

//#ifdef QT_OPENGL_LIB
#   include "../../../glew/include/GL/glew.h"
#   include "iOpenGLHelper.h"
#   include "iArray.h"
#   include "iChannel.h"


/**
 * Touch description.
 */
struct sTouchData : public sTouch {
    IArrayGridPatch * patch;
};

/**
 * OpenGL-based channel implementation.
 */
class Channel_gl : public IChannel,
                   public CRefCounter
{
    _GLEW_ENABLED

protected:
    /**
     * Channel identifier.
     */
    FOURCC m_Uid;

    int          m_Id;
    int          m_LastPatchSize_x;
    int          m_LastPatchSize_y;
    int          m_StackPosition;
    int          m_LayerDpi;
    bool         m_Changed;
    bool         m_GLinited;

    /**
     * Channel data as a patch grid.
     */
     IArrayGrid * m_Data;

     IOpenGLShader * m_glslCopy;
     IOpenGLShader * m_glslTouch;
     IOpenGLShader * m_glslFeedback;

     // A bunch of OpenGL variables.
     //
     GLuint m_glPar0_src;
     GLuint m_glPar1_primitive;
     GLuint m_glPar1_color;
     GLuint m_glPar1_paper;
     GLuint m_glPar1_oldpatch;
     GLuint m_glPar1_controlmask;
     GLuint m_glPar1_sensitivity;
     GLuint m_glPar1_noise_freq;
     GLuint m_glPar1_noise_amp;
     GLuint m_glPar1_bleedrate;
     GLuint m_glPar1_suckrate;
     GLuint m_glPar1_thickness;
     GLuint m_glPar1_channel_mu;
     GLuint m_glPar1_noise_re;
     GLuint m_glPar1_noise_im;
     GLuint m_glPar1_paper_txmu;
     GLuint m_glPar1_paper_txadd;
     GLuint m_glPar1_patch_txmu;
     GLuint m_glPar1_patch_txadd;
     GLuint m_glPar1_color_txmu;
     GLuint m_glPar1_color_txadd;
     GLuint m_glPar1_world_txmu;
     GLuint m_glPar1_world_txadd;
     GLuint m_glPar1_brush_uv;
     GLuint m_glPar1_control;
     GLuint m_glPar2_mask;
     GLuint m_glPar2_patch;
     GLuint m_glPar2_color;
     GLuint m_glPar2_color_txmu;
     GLuint m_glPar2_color_txadd;
     GLuint m_glPar2_suckrate;
     GLuint m_glPar2_exhaustrate;
     IOpenGLHelper * m_Helper;

     void beginGl     (void);
     void endGl       (void);
     void checkInited (void);

      Channel_gl (FOURCC uid, int width, int height, IOpenGLHelper * helper);
      virtual ~Channel_gl (void);

      /**
        * Touch method for an optical channel.
        */
      void touch_0 (IInstrumentLockEx * operation,
                    const sTouch * stroke,
                    int length);

      /**
        * Touch method for a meta channel.
        */
      void touch_1 (IInstrumentLockEx * operation,
                    const sTouch * stroke,
                    int length);

      virtual void suicide (void) {
          delete this;
      }

friend IChannel * g_fac_SPAWN_CHANNEL (FOURCC, int, int, IOpenGLHelper *);
public:
    /**
     *  Channel type identifier.
     */
    virtual FOURCC uid (void) {
        return m_Uid;
    }

    /**
     * Get the position of this channel
     * in a channel stack.
     */
    virtual int stackPosition (void) {
        return m_StackPosition;
    }

    /**
     * Apply the given instrument to the
     * channel data at the given positions
     */
    virtual void touch (IInstrumentLockEx * operation,
                        const sTouch * stroke,
                        int length);

    /**
     * Remove all data fromm channel.
     */
    virtual void clear (void);

    /**
     * Lock the contents
     * of this channel.
     */
    virtual IArrayGrid * lock (void) {
        // FIXME Lock!
        return m_Data;
    }

    /**
     * Unlock he channel.
     */
    virtual void unlock (void) {

    }

    /**
     * Get the width of a channel buffer.
     */
    virtual int width (void) {
        if (m_Data) {
            return m_Data->width ();
        } else {
            return 0;
        }
    }

    /**
     * Get the height of the channelbuffer.
     */
    virtual int height (void) {
        if (m_Data) {
            return m_Data->height ();
        } else {
            return 0;
        }
    }

    /**
     * Resize the channel.
     */
    virtual void resize (int width, int height);

    /**
     * Set the layer dpi for paper texture scaling.
     */
    virtual void setLayerDpi (int dpi) {
        m_LayerDpi = dpi;
    }

    /**
     * Reload all OpenGL shaders.
     */
    virtual void reloadShaders (void);

    /********* IDATACONTAINER IMPLANTS ************/

    /**
     * Check if the container's content
     * was changed since the last commit.
     */
    virtual bool changesCheck (void) const {
        return m_Changed;
    }

    /**
     * Commit the changes. This resets the
     * result returned by changesCheck ().
     */
    virtual void changesCommit (void) {
        m_Changed = FALSE;
    }

    /**
     * The length of buffer required
     * to save the cantainer's data.
     */
    virtual unsigned long requiredBufferSize (void) const;

    /**
     * Save the container's data into
     * the external buffer.
     * @return Number of bytes actually written.
     */
    virtual unsigned long saveData (QDataStream * stream, unsigned long capacity);

    /**
     * Load the data from  the external
     * buffer into container.
     * @return Number of bytes actually read.
     */
    virtual unsigned long loadData (QDataStream * stream, unsigned long capacity);

    /******** ISPAWNEDOBJECT IMPLANTS ***********/

    /**
     * Get the classname for this object.
     */
    virtual const char * myClassName (void) {
        return "Server::Channel_gl";
    }

    /**
     * Get the Id of current allocated object.
     */
    virtual int objectId (void) {
        return m_Id;
    }

    virtual void addRef (void) {
        CRefCounter::addRef ();
    }

    virtual void release (void) {
        CRefCounter::release ();
    }

    /**
     * Ask the object to emit all
     * notification signals if any.
     */
    virtual void pokeObject (void) {

    }
};

//#endif
