/***************************************************************************
 *   Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>              *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "instrument_brushes.h"
#include "simplebrush.h"
#include "tool.h"

Instrument_brushes::Instrument_brushes (void) {
    m_Logger = NULL;
    m_Id     = ISpawnedObject::allocateId ();

    // Not registering static objects?
    //ISpawnedObject::_register (this);

    Q_INIT_RESOURCE (brushes);
}

Instrument_brushes::~Instrument_brushes (void) {
    //ISpawnedObject::_unregister (this);

    SAFE_RELEASE (m_Logger);
}

IInstrument * Instrument_brushes::create (int index, IOpenGLHelper * helper) {
    if (index >= 0 && index < INSTRUMENTS_IN_PACK)
        return dynamic_cast<IInstrument*>(new BrushInstrument (index, this, helper, m_Logger));
    else
    if (index >= INSTRUMENTS_IN_PACK && index < INSTRUMENTS_IN_PACK + INSTRUMENTS_IN_TOOLPACK)
        return dynamic_cast<IInstrument*>(new ToolInstrument (index - INSTRUMENTS_IN_PACK,
                                                              this, helper, m_Logger));

    return NULL;
}

IInstrument * Instrument_brushes::create (const char * name, IOpenGLHelper * helper) {
    extern const char * g_CLASSES_br[INSTRUMENTS_IN_PACK +
                                     INSTRUMENTS_IN_TOOLPACK];

    for (int i = 0; i < INSTRUMENTS_IN_PACK; i++) {
        if (0 == strcmp (name, g_CLASSES_br[i]))
            return dynamic_cast<IInstrument*>(new BrushInstrument (i, this, helper, m_Logger));
    }

    for (int i = 0; i < INSTRUMENTS_IN_TOOLPACK; i++) {
        if (0 == strcmp (name, g_CLASSES_br[INSTRUMENTS_IN_PACK + i]))
            return dynamic_cast<IInstrument*>(new ToolInstrument (i - INSTRUMENTS_IN_PACK,
                                                                  this, helper, m_Logger));
    }

    return NULL;
}

Q_EXPORT_PLUGIN2 (brushes, Instrument_brushes);

