/***************************************************************************
 *   Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>              *
 *                                                                         *
 *   This file is a part of QAquarelle project.                            *
 *                                                                         *
 *   QAquarelle is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#pragma once
#include "commtypes.h"
#include "commdefs.h"
#include "commmath.h"
#include "refcounter.h"

#include "qhasmath.h"

#define BRUSH_MAX_SKEL_LENGTH       64
#define BRUSH_MAX_NODE_LINKS        8
#define MAX_PRIMITIVE_VERTICES      32


/**
 * A single skeleton
 * node.
 */
struct sTuftNode : public sPoint3f {
    float x0, y0, z0;

    int   nlinks;
    bool  fixed;
    bool  spine;

    float       length0[BRUSH_MAX_NODE_LINKS];
    float       length1[BRUSH_MAX_NODE_LINKS];
    sTuftNode * linkage[BRUSH_MAX_NODE_LINKS];
};

class BrushTuft {
    /**
     * Skeleton of the instrument.
     * as the linked nodes.
     */
    sTuftNode m_Skeleton [BRUSH_MAX_SKEL_LENGTH];

    float vtx_output[MAX_PRIMITIVE_VERTICES * 7];
    float vtx_backup[MAX_PRIMITIVE_VERTICES * 7];

    int   m_SkelLength;
    float m_Radius;
    float m_Elasticity;
    float m_TotalAngle;
    float m_Friction_angle;
    float m_Tilt_angle;
    float m_Tilt_modulus;
    float m_Ground_y;

    //float f  (void) const;
    //void  df (const float * x, float * result) const;

    float lerp_vtx (float a, float b) const;

public:
    BrushTuft (void);

    /**
     * Add a new node.
     */
    sTuftNode * addNode (float x, float y, float z, bool spine = FALSE, bool fixed = FALSE);

    /**
     * Link two nodes together.
     */
    void linkNodes (sTuftNode * node0, sTuftNode * node1);

    void setSize (float sz) {
        m_Radius = 0.5f * sz;
    }

    void setElastisity (float e) {
        m_Elasticity = e;
    }

    void setGround   (float y);
    void setFriction (float fx, float fy);
    void setTilt     (float tx, float ty);

    /**
     * Estimate the position of
     * all nodes.
     */
    void validate (void);

    /**
     * Restore the initial configuration.
     */
    void reset (void);

    /**
     * Get the brush geometry
     * to be written into VBO.
     */
    const float * getGeometry (int & nVertices);

    float getAngle (void) {
        return m_TotalAngle;
    }
};
